@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -7,6 +7,7 @@
 #  License, use of this software will be governed by the Apache License, Version 2.0,
 #  included in the file licenses/APL.txt.
 
+from __future__ import annotations
 
 from typing import overload
 
@@ -20,99 +21,114 @@ from synnax.channel.payload import (
     ChannelNames,
     ChannelParams,
     ChannelPayload,
+    Operation,
     normalize_channel_params,
 )
 from synnax.channel.retrieve import ChannelRetriever
 from synnax.channel.writer import ChannelWriter
-from synnax.exceptions import NotFoundError, MultipleFoundError, ValidationError
-from synnax.framer.client import Client
+from synnax.exceptions import MultipleFoundError, NotFoundError, ValidationError
+from synnax.framer.client import Client as FrameClient
+from synnax.ontology.payload import ID
 from synnax.telem import (
     CrudeDataType,
-    CrudeRate,
     CrudeTimeStamp,
     DataType,
-    Rate,
+    MultiSeries,
     Series,
     TimeRange,
 )
+from synnax.util.normalize import normalize
+
+CHANNEL_ONTOLOGY_TYPE = ID(type="channel")
 
 
 class Channel(ChannelPayload):
     """A channel is a logical collection of samples emitted by or representing the
-    values of a single source. See https://docs.synnaxlabs.com/concepts/channels for an
-    introduction to channels and how they work.
+    values of a single source. See
+    https://docs.synnaxlabs.com/reference/concepts/channels for an introduction to
+    channels and how they work.
     """
 
-    ___frame_client: Client | None = PrivateAttr(None)
-
-    class Config:
-        arbitrary_types_allowed = True
+    ___frame_client: FrameClient | None = PrivateAttr(None)
+    __client: ChannelClient | None = PrivateAttr(None)
 
     def __init__(
         self,
         *,
         name: str,
         data_type: CrudeDataType,
-        rate: CrudeRate = 0,
         is_index: bool = False,
         index: ChannelKey = 0,
         leaseholder: int = 0,
         key: ChannelKey = 0,
+        virtual: bool | None = None,
         internal: bool = False,
-        _frame_client: Client | None = None,
+        expression: str = "",
+        operations: list[Operation] | None = None,
+        _frame_client: FrameClient | None = None,
+        _client: ChannelClient | None = None,
     ) -> None:
         """Initializes a new Channel using the given parameters. It's important to note
         that this does not create the Channel in the cluster. To create the channel,
         call client.channels.create(channel).
 
-        :param data_type: The data type of the samples in the channel e.g. np.int64
-        :param rate: Rate sets the rate at which the channels values are written. If
-        this parameter is non-zero, is_index must be false and index must be an empty
-        string or unspecified.
-        :param name: A human-readable name for the channel.
-        :param key: Is auto-assigned by the cluster, and should not be set by the
-        caller.
+        :param name: A name for the channel.
+        :param data_type: The data type of the samples in the channel. For example, `"float32"`.
         :param is_index: Boolean indicating whether the channel is an index. Index
-        channels should have ax data type of synnax.TIMESTAMP.
+        channels should have a data type of synnax.TIMESTAMP.
         :param index: The key of the channel that indexes this channel.
-        :param leaseholder: The node that holds the lease for this channel. If you
-        don't know what this is, leave it at the default value of 0.
+        :param leaseholder: The node that holds the lease for this channel. If you don't know
+        what this is, leave it at the default value of 0.
+        :param virtual: Boolean indicating whether the channel is virtual. Virtual
+        channels do not store any data, and are used for streaming purposes only.
+        :param expression: An optional Lua expression that defines the channel as a
+        calculation of another channel. If this is set, the channel will be
+        automatically configured as virtual.
+        :param internal: Boolean indicating whether the channel is internal. Internal
+        channels are not visible to the user and are used for internal purposes only.
+        :param operations: A list of operations to apply to the channel. Operations
+        include aggregations like min, max, avg over a time duration or triggered by
+        a reset channel.
+        :returns: The created channel.
         :param _frame_client: The backing client for reading and writing data to and
         from the channel. This is provided by the Synnax py during calls to
         .channels.create() and .channels.retrieve() and should not be set by the caller.
         """
+        if virtual is None:
+            virtual = len(expression) > 0
         super().__init__(
             data_type=DataType(data_type),
-            rate=Rate(rate),
             name=name,
             leaseholder=leaseholder,
             key=key,
             is_index=is_index,
             index=index,
             internal=internal,
+            virtual=virtual,
+            expression=expression,
+            operations=operations,
         )
         self.___frame_client = _frame_client
+        self.__client = _client
 
     @overload
     def read(
         self,
         start_or_range: TimeRange,
-    ) -> Series:
-        ...
+    ) -> MultiSeries: ...
 
     @overload
     def read(
         self,
         start_or_range: CrudeTimeStamp,
         end: CrudeTimeStamp,
-    ) -> Series:
-        ...
+    ) -> MultiSeries: ...
 
     def read(
         self,
         start_or_range: CrudeTimeStamp | TimeRange,
         end: CrudeTimeStamp | None = None,
-    ) -> Series:
+    ) -> MultiSeries:
         """Reads telemetry from the channel between the two timestamps.
 
         :param start_or_range: The starting timestamp of the range to read from.
@@ -132,10 +148,18 @@ class Channel(ChannelPayload):
         :param data: The telemetry to write to the channel.
         :returns: None.
         """
-        self.__frame_client.write(start, data, self.key)
+        self.__frame_client.write(start, self.key, data)
+
+    def rename(self, name: ChannelName) -> None:
+        """Renames the channel.
+
+        :param name: The new name for the channel.
+        :returns: None.
+        """
+        self.__client.rename(self.key, name)
 
     @property
-    def __frame_client(self) -> Client:
+    def __frame_client(self) -> FrameClient:
         if self.___frame_client is None:
             raise ValidationError(
                 "Cannot read from or write to channel that has not been created."
@@ -151,25 +175,28 @@ class Channel(ChannelPayload):
     def to_payload(self) -> ChannelPayload:
         return ChannelPayload(
             data_type=self.data_type,
-            rate=self.rate,
             name=self.name,
             leaseholder=self.leaseholder,
             key=self.key,
             index=self.index,
             is_index=self.is_index,
+            virtual=self.virtual,
+            internal=self.internal,
+            expression=self.expression,
+            operations=self.operations,
         )
 
 
 class ChannelClient:
     """The core py class for executing channel operations against a Synnax cluster."""
 
-    _frame_client: Client
+    _frame_client: FrameClient
     _retriever: ChannelRetriever
     _creator: ChannelWriter
 
     def __init__(
         self,
-        frame_client: Client,
+        frame_client: FrameClient,
         retriever: ChannelRetriever,
         creator: ChannelWriter,
     ):
@@ -187,25 +214,24 @@ class ChannelClient:
         *,
         data_type: CrudeDataType = DataType.UNKNOWN,
         name: ChannelName = "",
-        rate: CrudeRate = Rate(0),
         index: ChannelKey = 0,
         is_index: bool = False,
         leaseholder: int = 0,
+        virtual: bool | None = None,
+        expression: str | None = None,
+        operations: list[Operation] | None = None,
         retrieve_if_name_exists: bool = False,
-    ) -> Channel:
-        ...
+    ) -> Channel: ...
 
     @overload
     def create(
         self, channels: Channel, *, retrieve_if_name_exists: bool = False
-    ) -> Channel:
-        ...
+    ) -> Channel: ...
 
     @overload
     def create(
         self, channels: list[Channel], *, retrieve_if_name_exists: bool = False
-    ) -> list[Channel]:
-        ...
+    ) -> list[Channel]: ...
 
     def create(
         self,
@@ -213,48 +239,63 @@ class ChannelClient:
         *,
         data_type: CrudeDataType = DataType.UNKNOWN,
         name: ChannelName = "",
-        rate: CrudeRate = Rate(0),
         is_index: bool = False,
         index: ChannelKey = 0,
         leaseholder: int = 0,
+        virtual: bool | None = None,
+        expression: str = "",
+        operations: list[Operation] | None = None,
         retrieve_if_name_exists: bool = False,
     ) -> Channel | list[Channel]:
-        """Creates a new channel or set of channels in the cluster. Possible arguments
-        are as follows:
+        """Creates new channel(s) in the Synnax cluster.
 
         Overload 1:
-        :param data_type: The data type of the samples in the channel e.g np.int64
-        :param rate: Rate sets the rate at which the channels values are written. If this
-        parameter is non-zero, is_index must be false and index must be an empty string or
-        unspecified.
-        :param name: A human-readable name for the channel.
+        :param data_type: The data type of the samples in the channel. For example, `"float32"`.
+        :param name: A name for the channel.
         :param is_index: Boolean indicating whether the channel is an index. Index
-        channels should have ax data type of synnax.TIMESTAMP.
-        :param index: The key or channel that indexes this channel.
+        channels should have a data type of synnax.TIMESTAMP.
+        :param index: The key of the channel that indexes this channel.
         :param leaseholder: The node that holds the lease for this channel. If you don't know
         what this is, leave it at the default value of 0.
+        :param virtual: Boolean indicating whether the channel is virtual. Virtual
+        channels do not store any data, and are used for streaming purposes only.
+        :param expression: An optional expression that defines the channel as a
+        calculation of another channel. If this is set, the channel will be
+        automatically configured as virtual.
+        set if expression is not an empty string. If expression is not an empty string,
+        this should have at least one channel.
+        :param retrieve_if_name_exists: Boolean indicating whether to retrieve channels
+        with the same name if they already exist in the cluster.
         :returns: The created channel.
 
         Overload 2:
 
         :param channels: A single channel to create.
+        :param retrieve_if_name_exists: Boolean indicating whether to retrieve channels
+        with the same name if they already exist in the cluster.
         :returns: The created channel.
 
         Overload 3:
 
         :param channels: A list of channels to create.
+        :param retrieve_if_name_exists: Boolean indicating whether to retrieve channels
+        with the same name if they already exist in the cluster.
         :returns: The created channels.
         """
 
         if channels is None:
+            if is_index and data_type == DataType.UNKNOWN:
+                data_type = DataType.TIMESTAMP
             _channels = [
                 ChannelPayload(
                     name=name,
                     leaseholder=leaseholder,
-                    rate=Rate(rate),
                     data_type=DataType(data_type),
                     index=index,
                     is_index=is_index,
+                    virtual=virtual if virtual is not None else len(expression) > 0,
+                    expression=expression,
+                    operations=operations,
                 )
             ]
         elif isinstance(channels, Channel):
@@ -275,59 +316,71 @@ class ChannelClient:
         return created if isinstance(channels, list) else created[0]
 
     @overload
-    def retrieve(self, channel: ChannelKey | ChannelName) -> Channel:
-        ...
+    def retrieve(self, channel: ChannelKey | ChannelName) -> Channel: ...
 
     @overload
     def retrieve(
         self,
         channel: ChannelKeys | ChannelNames,
-    ) -> list[Channel]:
-        ...
+    ) -> list[Channel]: ...
 
     def retrieve(self, channel: ChannelParams) -> Channel | list[Channel]:
         """Retrieves a channel or set of channels from the cluster.
 
         Overload 1:
 
-        :param key: The key of the channel to retrieve. If this is specified, the name
-        parameter is ignored.
-        :param name: The name of the channel to retrieve. If key is specified, this is
-        ignored.
-        Only one of key or name must be specified.
+        :param channel: The key or name of the channel to retrieve. If this is
         :returns: The associated channel.
         :raises QueryError: If the channel is not found.
 
         Overload 2 + 3:
-        :param keys: The keys of the channels to retrieve. If this is specified, the names
-        parameter is ignored.
-        :param names: The names of the channels to retrieve. If keys are specified, this is
-        ignored.
-        Only one of keys or names may be specified.
-        :param leaseholder: The node that holds the lease for the channels to retrieve. If you
-        don't know what this is, don't specify it.
-        :param include_not_found: Boolean indicating whether or not to include the keys or
-        names of the channels that were not found in the result.
-        :returns: The retrieved channels if include_not_found is False, otherwise a tuple
-        containing the retrieved channels and the keys or names of the channels that were
-        not found.
+        :param channel: The list of keys or the list of names for the channels to retrieve.
+        :returns: The retrieved channels.
         """
         normal = normalize_channel_params(channel)
         res = self._retriever.retrieve(channel)
         sug = self.__sugar(res)
         if not normal.single:
             return sug
-
         if len(res) == 1:
             return sug[0]
-
         if len(res) > 1:
             raise _multiple_results_error(channel, res)
-
         raise NotFoundError(f"Channel matching '{channel}' not found.")
 
+    @overload
+    def rename(self, keys: ChannelKey, names: ChannelName) -> None:
+        """Renames a channel in the cluster.
+
+        :param keys: The key of the channel to rename.
+        :param names: The new name for the channel.
+        :returns: None.
+        """
+        ...
+
+    @overload
+    def rename(self, keys: ChannelKeys, names: ChannelNames) -> None:
+        """Renames one or more channels in the cluster.
+
+        :param keys: The keys of the channels to rename.
+        :param names: The new names for the channels.
+        :returns: None.
+        """
+
+    def rename(self, keys: ChannelKeys, names: ChannelNames) -> None:
+        """Renames one or more channels in the cluster.
+
+        :param keys: The keys of the channels to rename.
+        :param names: The new names for the channels.
+        :returns: None.
+        """
+        self._creator.rename(normalize(keys), normalize(names))
+
     def __sugar(self, channels: list[ChannelPayload]) -> list[Channel]:
-        return [Channel(**c.dict(), _frame_client=self._frame_client) for c in channels]
+        return [
+            Channel(**c.model_dump(), _frame_client=self._frame_client)
+            for c in channels
+        ]
 
 
 def _multiple_results_error(
