@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -64,8 +64,7 @@ class SugaredConsole:
         choices: list[R] | None = None,
         password: bool = False,
         **kwargs: Unpack[AskKwargs[R]],
-    ) -> R:
-        ...
+    ) -> R: ...
 
     @overload
     def ask(
@@ -75,8 +74,7 @@ class SugaredConsole:
         choices: list[str] | None = None,
         password: bool = False,
         **kwargs: Unpack[DefaultAskKwargs[R]],
-    ) -> R:
-        ...
+    ) -> R: ...
 
     @overload
     def ask(
@@ -134,8 +132,7 @@ class SugaredConsole:
     @overload
     def select(
         self, rows: list[R], type_: type[R] = str, **kwargs: Unpack[DefaultAskKwargs[R]]
-    ) -> tuple[R, int]:
-        ...
+    ) -> tuple[R, int]: ...
 
     @overload
     def select(
@@ -145,8 +142,7 @@ class SugaredConsole:
         columns: list[str] | None = None,
         key: str | None = None,
         **kwargs: Unpack[DefaultAskKwargs[R]],
-    ) -> tuple[R, int]:
-        ...
+    ) -> tuple[R, int]: ...
 
     @overload
     def select(
@@ -155,8 +151,7 @@ class SugaredConsole:
         type_: type[R],
         columns: list[str] | None = None,
         **kwargs: Unpack[AskKwargs[R]],
-    ) -> tuple[R, int]:
-        ...
+    ) -> tuple[R, int]: ...
 
     @overload
     def select(
@@ -165,8 +160,7 @@ class SugaredConsole:
         type_: type[R],
         columns: list[str] | None = None,
         **kwargs: Unpack[NoneDefaultAskKwargs[R]],
-    ) -> tuple[R | None, int | None]:
-        ...
+    ) -> tuple[R | None, int | None]: ...
 
     def select(
         self,
@@ -197,10 +191,11 @@ class SugaredConsole:
         default_idx = 0
         no_cols = columns is None
         _columns = columns or list()
-        for row in rows:
+
+        for i, row in enumerate(rows):
             if isinstance(row, dict):
                 is_default = row[_key] == default
-                _rows.append(row)
+                _rows.append({"choice": str(i), **row})
                 if no_cols:
                     for k in row.keys():
                         if k not in _columns:
@@ -210,13 +205,16 @@ class SugaredConsole:
                 key = "value"
                 if len(_columns) > 0:
                     key = _columns[0]
-                _rows.append({key: row})
+                _rows.append({"choice": str(i), key: row})
             if is_default:
                 default_idx = len(_rows) - 1
 
         if len(_columns) == 0:
             _columns = ["value"]
 
+        if "choice" not in _columns:
+            _columns = ["choice"] + _columns
+
         if should_return:
             return v, default_idx
 
