@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -7,12 +7,10 @@
 #  License, use of this software will be governed by the Apache License, Version 2.0,
 #  included in the file licenses/APL.txt.
 
-from typing import Callable, Generic, TypeVar
+from collections.abc import Callable
 
 from synnax.cli.console import Console, SugaredConsole
 
-T = TypeVar("T")
-
 
 class Context:
     console: SugaredConsole
@@ -23,7 +21,7 @@ class Context:
         )
 
 
-class Flow(Generic[T]):
+class Flow[T]():
     steps: dict[str, Callable[[Context, T], str | None]]
     context: Context
 
