@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -6,6 +6,7 @@
 #  As of the Change Date specified in that file, in accordance with the Business Source
 #  License, use of this software will be governed by the Apache License, Version 2.0,
 #  included in the file licenses/APL.txt.
+
 import time
 from pathlib import Path
 from typing import Any
@@ -25,6 +26,7 @@ from synnax.cli.connect import connect_client
 from synnax.cli.flow import Context, Flow
 from synnax.cli.io import prompt_file
 from synnax.cli.telem import select_data_type
+from synnax.cli.warning import warning
 from synnax.ingest.row import RowIngestionEngine
 from synnax.io import IO_FACTORY, ChannelMeta, IOFactory, ReaderType, RowFileReader
 from synnax.telem import DataType, Rate, TimeRange, TimeStamp
@@ -34,8 +36,10 @@ GROUP_ALL = "__all__"
 
 
 @click.command()
+@click.pass_context
 @click.argument("path_", type=click.Path(exists=True), required=False, default=None)
-def ingest(path_: str | None):
+def ingest(ctx: click.Context, path_: str | None) -> None:
+    warning(ctx)
     return pure_ingest(path_)
 
 
@@ -49,6 +53,7 @@ def pure_ingest(
     flow.add("connect_client", _connect_client)
     flow.add("ingest_all", ingest_all)
     flow.add("channels_to_ingest", channels_to_ingest)
+    flow.add("skip_invalid_channels", skip_invalid_channels)
     flow.add("validate_channels_exist", validate_channels_exist)
     flow.add("validate_data_types", validate_data_types)
     flow.add("validate_start_time", validate_start_time)
@@ -86,6 +91,7 @@ def run_ingestion(ctx: Context, cli: IngestionCLI) -> None:
     assert cli.db_channels is not None
     assert cli.client is not None
     assert cli.start is not None
+    filtered_reader = initialize_reader(ctx, cli)
     if cli.reader.type() == ReaderType.Row:
         engine = RowIngestionEngine(cli.client, cli.reader, cli.db_channels, cli.start)
     else:
@@ -121,7 +127,7 @@ def ingest_all(ctx: Context, cli: IngestionCLI) -> str | None:
     assert cli.reader is not None
     if ctx.console.ask("Would you like to ingest all channels?", default=True):
         cli.filtered_channels = cli.reader.channels()
-        return "validate_channels_exist"
+        return "skip_invalid_channels"
     else:
         return "channels_to_ingest"
 
@@ -136,6 +142,24 @@ def channels_to_ingest(ctx: Context, cli: IngestionCLI) -> str | None:
         return None
     all_names = [v for l in grouped.values() for v in l]
     cli.filtered_channels = [ch for ch in channels if ch.name in all_names]
+    return "skip_invalid_channels"
+
+
+def skip_invalid_channels(ctx: Context, cli: IngestionCLI) -> str | None:
+    assert cli.reader is not None
+    data_types = {
+        key: dt
+        for key, dt in read_data_types(ctx, cli).items()
+        if dt == DataType.UNKNOWN
+    }
+    if len(data_types) > 0:
+        ctx.console.info("The following channels have non-numeric data types")
+        channel_name_table(ctx, [ch for ch in data_types.keys()])
+        if not ctx.console.ask("Skip these channels?", default=True):
+            return None
+        cli.filtered_channels = [
+            ch for ch in cli.filtered_channels if ch.name not in data_types.keys()
+        ]
     return "validate_channels_exist"
 
 
@@ -183,7 +207,10 @@ def read_data_types(ctx: Context, cli: IngestionCLI) -> dict[str, DataType]:
         if len(samples) == 0:
             ctx.console.warn(f"Channel {ch.name} has no samples")
             continue
-        data_types[ch.name] = DataType(samples.to_numpy().dtype)
+        try:
+            data_types[ch.name] = DataType(samples.to_numpy().dtype)
+        except TypeError:
+            data_types[ch.name] = DataType.UNKNOWN
     return data_types
 
 
@@ -391,7 +418,6 @@ def create_channels(ctx: Context, cli: IngestionCLI) -> str | None:
                     name=ch.name,
                     is_index=False,
                     index="" if is_rate else rate_or_index,
-                    rate=rate_or_index if is_rate else 0,
                     data_type=[dt for dt, chs in dt_grouped.items() if ch in chs][0],
                 )
             )
