@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -9,16 +9,17 @@
 
 import click
 
+from .check_timing import check_timing
 from .ingest import ingest
 from .login import login
 from .ts_convert import tsconvert
 
 
 @click.group()
-def synnax():
-    ...
+def synnax() -> None: ...
 
 
 synnax.add_command(ingest)
 synnax.add_command(login)
 synnax.add_command(tsconvert)
+synnax.add_command(check_timing)
