@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -8,4 +8,4 @@
 #  included in the file licenses/APL.txt.
 
 from synnax.control.client import Client
-from synnax.control.controller import Controller
+from synnax.control.controller import Controller, ScheduledCommand
