@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -9,6 +9,6 @@
 
 from synnax.framer.client import Client
 from synnax.framer.frame import Frame
-from synnax.framer.iterator import Iterator
+from synnax.framer.iterator import AUTO_SPAN, Iterator
 from synnax.framer.streamer import AsyncStreamer, Streamer
-from synnax.framer.writer import BufferedWriter, Writer, WriterMode
+from synnax.framer.writer import Writer, WriterMode
