@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -53,7 +53,9 @@ class RowIngestionEngine:
         self.reader = reader
         self.client = client
         self.reader.set_chunk_size(self.get_chunk_size())
-        self.writer = self.client.open_writer(start, [ch.key for ch in channels])
+        self.writer = self.client.open_writer(
+            start, [ch.key for ch in channels], err_on_extra_chans=False
+        )
         self.end = start
 
     def get_chunk_size(self):
@@ -78,10 +80,10 @@ class RowIngestionEngine:
                         chunk = self.reader.read()
                         self._write(chunk)
                         tp = chunk.size / (datetime.now() - t0).total_seconds()
-                        progress.update(task, advance=chunk.size, tp=tp)
+                        progress.update(task, advance=chunk.size, tp=int(tp))
                     except StopIteration:
                         break
-            self.end, _ = self.writer.commit()
+            self.end = self.writer.commit()
         finally:
             self.reader.close()
             self.writer.close()
