@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -7,13 +7,14 @@
 #  License, use of this software will be governed by the Apache License, Version 2.0,
 #  included in the file licenses/APL.txt.
 
+import csv
+from collections.abc import Iterator
 from pathlib import Path
-from typing import Iterator
 
 import pandas as pd
 from pandas.io.parsers import TextFileReader
 
-from synnax.exceptions import ValidationError
+from synnax.io.df_converter import convert_df
 from synnax.io.matcher import new_extension_matcher
 from synnax.io.protocol import ChannelMeta, FileWriter, ReaderType, RowFileReader
 
@@ -47,13 +48,19 @@ class CSVReader(CSVMatcher):  # type: ignore
     ):
         self._path = path
         self.channel_keys = keys
-        self.chunk_size = chunk_size or int(5e5)
+        self.chunk_size = chunk_size or int(1e4)
         self._channels = None
         self._row_count = None
         self._skip_rows = 0
         self._calculated_skip_rows = False
         self.__reader = None
 
+    def __detect_delimiter(self) -> str:
+        with open(self._path, "r") as file:
+            sample = file.read(1024)
+            dialect = csv.Sniffer().sniff(sample)
+            return dialect.delimiter
+
     def seek_first(self):
         self.close()
         self.__reader = pd.read_csv(
@@ -62,6 +69,7 @@ class CSVReader(CSVMatcher):  # type: ignore
             usecols=self.channel_keys,
             header=0,
             skiprows=self.__get_skip_rows(),
+            delimiter=self.__detect_delimiter(),
         )
 
     def __get_skip_rows(self) -> int | tuple[int, int]:
@@ -72,6 +80,7 @@ class CSVReader(CSVMatcher):  # type: ignore
             self._path,
             chunksize=1,
             usecols=self.channel_keys,
+            delimiter=self.__detect_delimiter(),
         )
         self._skip_rows = 0
 
@@ -79,14 +88,16 @@ class CSVReader(CSVMatcher):  # type: ignore
             try:
                 df = next(r)
             except StopIteration:
-                raise ValidationError("No valid data found in CSV file")
-
-            # check if the first value is a string
+                # If we reach the end of the file, just do a best effort calculation
+                self._skip_rows = 0
+                break
             if isinstance(df.iloc[0, 0], str):
                 self._skip_rows += 1
             else:
                 self._calculated_skip_rows = True
 
+        self._calculated_skip_rows = True
+
         r.close()
         if self._skip_rows > 0:
             return 1, self._skip_rows
@@ -94,7 +105,9 @@ class CSVReader(CSVMatcher):  # type: ignore
 
     def channels(self) -> list[ChannelMeta]:
         if not self._channels:
-            cols = pd.read_csv(self._path, nrows=0).columns
+            cols = pd.read_csv(
+                self._path, nrows=0, delimiter=self.__detect_delimiter()
+            ).columns
             self._channels = [
                 ChannelMeta(name=name.strip(), meta_data=dict()) for name in cols
             ]
@@ -104,10 +117,10 @@ class CSVReader(CSVMatcher):  # type: ignore
         self.chunk_size = chunk_size
 
     def read(self) -> pd.DataFrame:
-        return next(self._reader)
+        return convert_df(next(self._reader))
 
     def __iter__(self) -> Iterator[pd.DataFrame]:
-        return self._reader.__iter__()
+        return CSVReaderIterator(self._reader)
 
     @classmethod
     def type(cls) -> ReaderType:
@@ -157,6 +170,8 @@ class CSVWriter(CSVMatcher):  # type: ignore
     ):
         self._path = path
         self._header = True
+        if path.exists():
+            path.unlink()
 
     # Doing a protocol implementation check here because
     # it's hard for pyright to handle factories that return
@@ -173,3 +188,16 @@ class CSVWriter(CSVMatcher):  # type: ignore
 
     def close(self):
         pass
+
+
+class CSVReaderIterator:
+    base: Iterator[pd.DataFrame]
+
+    def __init__(self, base: Iterator[pd.DataFrame]):
+        self.base = base
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        return convert_df(next(self.base))
