@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -7,9 +7,10 @@
 #  License, use of this software will be governed by the Apache License, Version 2.0,
 #  included in the file licenses/APL.txt.
 
+from collections.abc import Iterator
 from enum import Enum
 from pathlib import Path
-from typing import Iterator, Protocol
+from typing import Protocol
 
 from pandas import DataFrame
 
@@ -66,8 +67,7 @@ class BaseReader(File, Protocol):
         path: Path,
         keys: list[str] | None = None,
         chunk_size: int | None = None,
-    ):
-        ...
+    ): ...
 
     def channels(self) -> list[ChannelMeta]:
         """:returns : a list of channel metadata for the file."""
@@ -87,7 +87,7 @@ class BaseReader(File, Protocol):
         ...
 
     def seek_first(self):
-        """Seeks the reader to the first  sampele in the file."""
+        """Seeks the reader to the first sample in the file."""
         ...
 
 
