@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -10,6 +10,7 @@
 from alamos import NOOP, Instrumentation
 from freighter import URL
 
+from synnax import PolicyClient
 from synnax.auth import AuthenticationClient
 from synnax.channel import ChannelClient
 from synnax.channel.retrieve import CacheChannelRetriever, ClusterChannelRetriever
@@ -17,15 +18,20 @@ from synnax.channel.writer import ChannelWriter
 from synnax.config import try_load_options_if_none_provided
 from synnax.control import Client as ControlClient
 from synnax.framer import Client
+from synnax.framer.deleter import Deleter
 from synnax.hardware.client import Client as HardwareClient
-from synnax.hardware.retrieve import Retriever as HardwareRetriever
-from synnax.hardware.writer import Writer as HardwareWriter
+from synnax.hardware.device import Client as DeviceClient
+from synnax.hardware.rack import Client as RackClient
+from synnax.hardware.task import Client as TaskClient
+from synnax.ontology import Client as OntologyClient
+from synnax.ontology.group import Client as GroupClient
 from synnax.options import SynnaxOptions
 from synnax.ranger import RangeRetriever, RangeWriter
 from synnax.ranger.client import RangeClient
 from synnax.signals.signals import Registry
 from synnax.telem import TimeSpan
 from synnax.transport import Transport
+from synnax.user.client import Client as UserClient
 
 
 class Synnax(Client):
@@ -46,17 +52,20 @@ class Synnax(Client):
             port=9090,
             username="synnax",
             password="seldon",
-            secure=True,
+            secure=True
         )
     """
 
     channels: ChannelClient
+    access: PolicyClient
+    user: UserClient
     ranges: RangeClient
     control: ControlClient
     signals: Registry
     hardware: HardwareClient
+    ontology: OntologyClient
 
-    __client: Transport
+    _transport: Transport
 
     def __init__(
         self,
@@ -70,6 +79,7 @@ class Synnax(Client):
         keep_alive: TimeSpan = TimeSpan.SECOND * 30,
         max_retries: int = 3,
         instrumentation: Instrumentation = NOOP,
+        cache_channels: bool = True,
     ):
         """Creates a new client. Connection parameters can be provided as arguments, or,
         if none are provided, the client will attempt to load them from the Synnax
@@ -95,20 +105,43 @@ class Synnax(Client):
             max_retries=max_retries,
             instrumentation=instrumentation,
         )
-        ch_retriever = CacheChannelRetriever(
-            ClusterChannelRetriever(self._transport.unary, instrumentation),
+        self.auth = AuthenticationClient(
+            transport=self._transport.unary,
+            username=opts.username,
+            password=opts.password,
+        )
+        self.auth.authenticate()
+        self._transport.use(self.auth.middleware())
+        self._transport.use_async(self.auth.async_middleware())
+
+        ch_retriever = ClusterChannelRetriever(self._transport.unary, instrumentation)
+        if cache_channels:
+            ch_retriever = CacheChannelRetriever(ch_retriever, instrumentation)
+        deleter = Deleter(self._transport.unary, instrumentation)
+        ch_creator = ChannelWriter(
+            self._transport.unary,
             instrumentation,
+            ch_retriever if cache_channels else None,
         )
-        ch_creator = ChannelWriter(self._transport.unary, instrumentation)
         super().__init__(
-            client=self._transport.stream,
+            stream_client=self._transport.stream,
             async_client=self._transport.stream_async,
+            unary_client=self._transport.unary,
             retriever=ch_retriever,
+            deleter=deleter,
+            instrumentation=instrumentation,
         )
+        groups = GroupClient(self._transport.unary)
+        self.ontology = OntologyClient(client=self._transport.unary, groups=groups)
         self.channels = ChannelClient(self, ch_retriever, ch_creator)
         range_retriever = RangeRetriever(self._transport.unary, instrumentation)
         range_creator = RangeWriter(self._transport.unary, instrumentation)
         self.signals = Registry(frame_client=self, channels=ch_retriever)
+        racks = RackClient(client=self._transport.unary)
+        tasks = TaskClient(
+            client=self._transport.unary, frame_client=self, rack_client=racks
+        )
+        devices = DeviceClient(client=self._transport.unary)
         self.ranges = RangeClient(
             unary_client=self._transport.unary,
             frame_client=self,
@@ -116,12 +149,14 @@ class Synnax(Client):
             writer=range_creator,
             retriever=range_retriever,
             signals=self.signals,
+            ontology=self.ontology,
+            tasks=tasks,
         )
         self.control = ControlClient(self, ch_retriever)
-        self.hardware = HardwareClient(
-            HardwareWriter(client=self._transport.unary),
-            HardwareRetriever(client=self._transport.unary),
-        )
+
+        self.hardware = HardwareClient(tasks=tasks, devices=devices, racks=racks)
+        self.access = PolicyClient(self._transport.unary, instrumentation)
+        self.user = UserClient(self._transport.unary)
 
     def close(self):
         """Shuts down the client and closes all connections. All open iterators or
@@ -149,13 +184,4 @@ def _configure_transport(
         keep_alive=keep_alive,
         max_retries=max_retries,
     )
-    if opts.username != "" or opts.password != "":
-        auth = AuthenticationClient(
-            transport=t.unary,
-            username=opts.username,
-            password=opts.password,
-        )
-        auth.authenticate()
-        t.use(*auth.middleware())
-        t.use_async(*auth.async_middleware())
     return t
