@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -10,14 +10,15 @@
 from __future__ import annotations
 
 import uuid
-from datetime import datetime, timedelta, timezone, tzinfo
+from datetime import UTC, datetime, timedelta, tzinfo
 from math import trunc
-from typing import ClassVar, Literal, TypeAlias, cast, get_args
+from typing import Any, ClassVar, Literal, TypeAlias, cast, get_args
 
 import numpy as np
 import pandas as pd
 from numpy.typing import DTypeLike
-from pydantic import BaseModel
+from pydantic import BaseModel, GetCoreSchemaHandler
+from pydantic_core import core_schema
 
 from synnax.exceptions import ContiguityError
 
@@ -38,12 +39,12 @@ class TimeStamp(int):
     * int - Treats the int as a nanosecond duration since the Unix epoch in UTC.
     """
 
-    def __new__(cls, value: CrudeTimeStamp):
+    def __new__(cls, value: CrudeTimeStamp) -> TimeStamp:
         if isinstance(value, str):
             value = int(value)
-        if isinstance(value, TimeStamp):
+        elif isinstance(value, TimeStamp):
             return value
-        if isinstance(value, TimeSpan):
+        elif isinstance(value, TimeSpan):
             value = int(value)
         elif isinstance(value, pd.Timestamp):
             value = int(
@@ -63,15 +64,16 @@ class TimeStamp(int):
         return super().__new__(cls, value)
 
     @classmethod
-    def __get_validators__(cls):
-        """Implemented for pydantic validation"""
-        yield cls.validate
+    def __get_pydantic_core_schema__(
+        cls, source_type: Any, handler: GetCoreSchemaHandler
+    ):
+        """Implemented for pydantic validation. Should not be used externally."""
+        return core_schema.no_info_after_validator_function(cls, handler(int))
 
     @classmethod
-    def validate(cls, v):
-        """Implemented for pydantic validation"""
-        # if its a string, cast the string to a number
-        return cls(v)
+    def validate(cls, value, *args, **kwargs):
+        """Implemented for pydantic validation. Should not be used externally."""
+        return cls(value)
 
     @classmethod
     def now(cls) -> TimeStamp:
@@ -110,11 +112,7 @@ class TimeStamp(int):
         :param tz: The timezone to use for the datetime. If not provided, the local
         timezone is used.
         """
-        return (
-            datetime.utcfromtimestamp(self / TimeSpan.SECOND)
-            .replace(tzinfo=timezone.utc)
-            .astimezone(tz)
-        )
+        return datetime.fromtimestamp(self / TimeSpan.SECOND, UTC).astimezone(tz)
 
     def after(self, ts: CrudeTimeStamp) -> bool:
         """:returns: True if the TimeStamp is strictly after the given TimeStamp.
@@ -185,7 +183,6 @@ class TimeStamp(int):
     MAX: TimeStamp
     """The maximum possible value of a TimeStamp"""
     ZERO: TimeStamp
-    """The zero value of a TimeStamp"""
 
 
 class TimeSpan(int):
@@ -214,12 +211,14 @@ class TimeSpan(int):
         return super().__new__(cls, value)
 
     @classmethod
-    def __get_validators__(cls):
+    def __get_pydantic_core_schema__(
+        cls, source_type: Any, handler: GetCoreSchemaHandler
+    ):
         """Implemented for pydantic validation. Should not be used externally."""
-        yield cls.validate
+        return core_schema.no_info_after_validator_function(cls, handler(int))
 
     @classmethod
-    def validate(cls, value):
+    def validate(cls, value, *args, **kwargs):
         """Implemented for pydantic validation. Should not be used externally."""
         return cls(value)
 
@@ -265,6 +264,30 @@ class TimeSpan(int):
             v += f"{nanos.nanoseconds}{TimeSpan.NANOSECOND_UNITS}"
         return v.strip()
 
+    @staticmethod
+    def from_seconds(span: CrudeTimeSpan) -> TimeSpan:
+        """An alternative constructor for TimeSpan that parses raw numeric values as
+        seconds instead of nanoseconds.
+        """
+        if isinstance(span, TimeSpan):
+            return span
+        if isinstance(span, Rate):
+            return span.period
+        if isinstance(span, (float, int)):
+            return TimeSpan.SECOND * span
+        return TimeSpan(span)
+
+    @staticmethod
+    def to_seconds(dur: CrudeTimeSpan | None) -> float | None:
+        if dur is None:
+            return None
+        return TimeSpan.from_seconds(dur).seconds
+
+    @property
+    def rate(self) -> Rate:
+        """:returns: The rate of the TimeSpan as a Rate."""
+        return Rate(1 / self.seconds)
+
     @property
     def days(self) -> float:
         """:returns: The decimal number of days in the TimeSpan, including fractional
@@ -381,7 +404,7 @@ class TimeSpan(int):
         return TimeSpan(super().__sub__(TimeSpan(rhs)))
 
     def __mul__(self, rhs: CrudeTimeSpan) -> TimeSpan:
-        return TimeSpan(super().__mul__(TimeSpan(rhs)))
+        return TimeSpan(self.nanoseconds * rhs)
 
     def __truediv__(self, rhs: CrudeTimeSpan) -> TimeSpan:
         return TimeSpan(super().__truediv__(TimeSpan(rhs)))
@@ -460,7 +483,7 @@ TimeSpan.HOUR = TimeSpan(60) * TimeSpan.MINUTE
 TimeSpan.HOUR_UNITS = "h"
 TimeSpan.DAY = TimeSpan.HOUR * 24
 TimeSpan.DAY_UNITS = "d"
-TimeSpan.MAX = TimeSpan(0xFFFFFFFFFFFFFFFF)
+TimeSpan.MAX = TimeSpan(2**63 - 1)
 TimeSpan.MIN = TimeSpan(0)
 TimeSpan.ZERO = TimeSpan(0)
 TimeSpan.UNITS = {
@@ -521,12 +544,14 @@ class Rate(float):
         return super().__new__(cls, value)
 
     @classmethod
-    def __get_validators__(cls):
+    def __get_pydantic_core_schema__(
+        cls, source_type: Any, handler: GetCoreSchemaHandler
+    ):
         """Implemented for pydantic validation. Should not be used externally."""
-        yield cls.validate
+        return core_schema.no_info_after_validator_function(cls, handler(float))
 
     @classmethod
-    def validate(cls, v):
+    def validate(cls, v, *args, **kwargs):
         """Implemented for pydantic validation. Should not be used externally."""
         return cls(v)
 
@@ -564,7 +589,7 @@ class Rate(float):
     def __str__(self):
         if self < 1:
             return f"{self.period} per cycle"
-        return str(int(self)) + "Hz"
+        return f"{round(self, 2)} Hz"
 
     def __repr__(self):
         return f"Rate({super().__repr__()} Hz)"
@@ -572,6 +597,9 @@ class Rate(float):
     def __mul__(self, rhs: CrudeRate) -> Rate:
         return Rate(super().__mul__(Rate(rhs)))
 
+    def __rmul__(self, other) -> Rate:
+        return self.__mul__(other)
+
     HZ: Rate
     """One Hz."""
     KHZ: Rate
@@ -621,12 +649,7 @@ class TimeRange(BaseModel):
         super().__init__(start=TimeStamp(start), end=TimeStamp(end))
 
     @classmethod
-    def __get_validators__(cls):
-        """Implemented for pydantic validation. Should not be used externally."""
-        yield cls.validate
-
-    @classmethod
-    def validate(cls, v):
+    def validate(cls, v, *args, **kwargs):
         """Implemented for pydantic validation. Should not be used externally."""
         if isinstance(v, TimeRange):
             return cls(v.start, v.end)
@@ -744,12 +767,14 @@ class Density(int):
         raise TypeError(f"Cannot convert {type(value)} to Density")
 
     @classmethod
-    def __get_validators__(cls):
+    def __get_pydantic_core_schema__(
+        cls, source_type: Any, handler: GetCoreSchemaHandler
+    ):
         """Implemented for pydantic validation. Should not be used externally."""
-        yield cls.validate
+        return core_schema.no_info_after_validator_function(cls, handler(int))
 
     @classmethod
-    def validate(cls, v):
+    def validate(cls, v, *args, **kwargs):
         """Implemented for pydantic validation. Should not be used externally."""
         return cls(v)
 
@@ -943,23 +968,24 @@ class DataType(str):
 
             raise TypeError(f"Cannot convert {type(value)} to DataType")
 
-        if np.issctype(value):
-            value = DataType._FROM_NUMPY.get(np.dtype(value), None)
-            if value is not None:
-                return value
-
         if isinstance(value, dict):
             return DataType.JSON
 
+        value = DataType._FROM_NUMPY.get(np.dtype(value), None)
+        if value is not None:
+            return value
+
         raise TypeError(f"Cannot convert {type(value)} to DataType")
 
     @classmethod
-    def __get_validators__(cls):
+    def __get_pydantic_core_schema__(
+        cls, source_type: Any, handler: GetCoreSchemaHandler
+    ):
         """Implemented for pydantic validation. Should not be used externally."""
-        yield cls.validate
+        return core_schema.no_info_after_validator_function(cls, handler(str))
 
     @classmethod
-    def validate(cls, v):
+    def validate(cls, v, *args, **kwargs):
         """Implemented for pydantic validation. Should not be used externally."""
         return cls(v)
 
@@ -968,6 +994,11 @@ class DataType(str):
         """Implemented for pydantic validation. Should not be used externally."""
         field_schema.update(type="string")
 
+    @classmethod
+    def __get_pydantic_json_schema__(cls, _schema_generator, _field_schema):
+        """Implemented for pydantic validation. Should not be used externally."""
+        return {"type": "string"}
+
     @property
     def np(self) -> np.dtype:
         """Converts a built-in DataType to a numpy type Scalar Type
@@ -979,10 +1010,8 @@ class DataType(str):
         return cast(np.dtype, npt)
 
     @property
-    def has_fixed_density(self) -> bool:
-        """:returns: True if the DataType has a fixed density"""
-        d = DataType._DENSITIES.get(self, None)
-        return d is not None and d != Density.UNKNOWN
+    def is_variable(self) -> bool:
+        return self == DataType.STRING or self == DataType.JSON
 
     @property
     def has_np(self) -> bool:
@@ -1075,7 +1104,22 @@ DataType._TO_NUMPY = {
     DataType.UINT16: np.dtype(np.uint16),
     DataType.UINT8: np.dtype(np.uint8),
 }
-DataType._FROM_NUMPY = {v: k for k, v in DataType._TO_NUMPY.items()}
+DataType._FROM_NUMPY = {
+    np.dtype(np.float64): DataType.FLOAT64,
+    np.dtype(np.float32): DataType.FLOAT32,
+    np.dtype(np.int64): DataType.INT64,
+    np.dtype(np.int32): DataType.INT32,
+    np.dtype(np.int16): DataType.INT16,
+    np.dtype(np.int8): DataType.INT8,
+    np.dtype(np.uint64): DataType.UINT64,
+    np.dtype(np.uint32): DataType.UINT32,
+    np.dtype(np.uint16): DataType.UINT16,
+    np.dtype(np.uint8): DataType.UINT8,
+    np.dtype(np.bool_): DataType.UINT8,
+    np.dtype(np.datetime64): DataType.TIMESTAMP,
+    np.dtype(np.str_): DataType.STRING,
+}
+
 DataType._DENSITIES = {
     DataType.UUID: Density.BIT128,
     DataType.FLOAT64: Density.BIT64,
@@ -1104,3 +1148,126 @@ class Bounds:
 
     def contains(self, value: float) -> bool:
         return self.lower < value <= self.upper
+
+
+class Alignment(int):
+    """Alignment is two array index values that can be used to represent
+    the location of a sample within an array of arrays. For example, if you have two arrays
+    that have 50 elements each, and you want the 15th element of the second array, you would
+    use Alignment(1, 15). The first index is called the 'domain index' and the second
+    index is called the 'sample index'. The domain index is the index of the array, and the
+    sample index is the index of the sample within that array.
+
+    You may think a better design is to just use a single number that overflows the arrays
+    before it, i.e., the value of our previous example would be 50 + 14 = 64. However, this
+    requires us to know the size of all arrays, which is not always possible.
+
+    While not as meaningful as a single number, Alignment is a uint64 that guarantees
+    that a larger value is, in fact, 'positionally' after a smaller value. This is useful
+    for ordering samples correctly.
+
+    The Alignment constructor accepts various types:
+
+    * int - Treats the int as a packed alignment value.
+    * tuple[int, int] - Treats as (domain_index, sample_index).
+    * Alignment - Returns a copy of the Alignment.
+    """
+
+    MAX_ALIGNMENT: ClassVar[int] = (1 << 64) - 1
+
+    def __new__(
+        cls, value: CrudeAlignment = 0, sample_idx: int | None = None
+    ) -> Alignment:
+        """Creates a new Alignment from various input types.
+
+        :param value: Either a packed int value, a tuple of (domain_idx, sample_idx),
+                      or an existing Alignment. Can also be a domain_idx if sample_idx is provided.
+        :param sample_idx: Optional sample index if value is provided as domain index.
+        :returns: A new Alignment.
+        """
+        if isinstance(value, Alignment):
+            return value
+        elif isinstance(value, tuple):
+            if len(value) != 2:
+                raise ValueError(
+                    f"Alignment tuple must have exactly 2 elements (domain_idx, sample_idx), got {len(value)}"
+                )
+            domain_idx, sample_idx = value
+            packed_value = (int(domain_idx) << 32) | int(sample_idx)
+        elif sample_idx is not None:
+            # Called as Alignment(domain_idx, sample_idx)
+            packed_value = (int(value) << 32) | int(sample_idx)
+        elif isinstance(value, (int, np.integer)):
+            # Packed value or zero
+            packed_value = int(value)
+        else:
+            raise TypeError(f"Cannot convert {type(value)} to Alignment")
+
+        return super().__new__(cls, packed_value)
+
+    @classmethod
+    def __get_pydantic_core_schema__(
+        cls, source_type: Any, handler: GetCoreSchemaHandler
+    ):
+        """Implemented for pydantic validation. Should not be used externally."""
+        return core_schema.no_info_after_validator_function(cls._validate, handler(int))
+
+    @classmethod
+    def _validate(cls, value: int) -> Alignment:
+        """Implemented for pydantic validation. Should not be used externally."""
+        # Use the constructor to handle all conversion logic
+        return cls(value)
+
+    @property
+    def domain_index(self) -> int:
+        """Returns the domain index of the Alignment. This is the index
+        in the array of arrays.
+
+        :returns: The domain index.
+        """
+        return int(self) >> 32
+
+    @property
+    def sample_index(self) -> int:
+        """Returns the sample index of the Alignment. This is the index within
+        a particular array.
+
+        :returns: The sample index.
+        """
+        return int(self) & 0xFFFFFFFF
+
+    def add_samples(self, samples: int) -> Alignment:
+        """Increments the sample index of the alignment.
+
+        :param samples: The number of samples to add.
+        :returns: A new Alignment with the sample index incremented.
+        """
+        return Alignment(self.domain_index, self.sample_index + samples)
+
+    def add(self, other: Alignment) -> Alignment:
+        """Adds another Alignment to this one.
+
+        :param other: The other Alignment to add.
+        :returns: A new Alignment with both domain and sample indices added.
+        """
+        return Alignment(
+            self.domain_index + other.domain_index,
+            self.sample_index + other.sample_index,
+        )
+
+    def __str__(self) -> str:
+        """Returns a string representation of the Alignment in the format 'domain-sample'.
+
+        :returns: A string representation of the Alignment.
+        """
+        return f"{self.domain_index}-{self.sample_index}"
+
+    def __repr__(self) -> str:
+        """Returns a string representation of the Alignment for debugging.
+
+        :returns: A string representation of the Alignment.
+        """
+        return f"Alignment({self.domain_index}, {self.sample_index})"
+
+
+CrudeAlignment: TypeAlias = int | tuple[int, int] | Alignment
