@@ -1,4 +1,4 @@
-#  Copyright 2023 Synnax Labs, Inc.
+#  Copyright 2025 Synnax Labs, Inc.
 #
 #  Use of this software is governed by the Business Source License included in the file
 #  licenses/BSL.txt.
@@ -7,12 +7,11 @@
 #  License, use of this software will be governed by the Apache License, Version 2.0,
 #  included in the file licenses/APL.txt.
 
-from typing import TypeVar
 
-T = TypeVar("T")
+from typing import Any
 
 
-def normalize(*args: T | tuple[T] | list[T]) -> list[T]:
+def normalize[T](*args: T | tuple[T] | list[T] | None) -> list[T]:
     """Flatten a list of lists into a single list.
 
     Args:
@@ -22,9 +21,30 @@ def normalize(*args: T | tuple[T] | list[T]) -> list[T]:
         A flattened list.
     """
     results: list[T] = list()
+    if args[0] is None:
+        return results
     for arg in args:
         if isinstance(arg, (list, tuple)):
             results.extend(arg)
-        else:
+        elif arg is not None:
             results.append(arg)
     return results
+
+
+def check_for_none(*args: Any) -> bool:
+    """Check if any of the arguments are None.
+
+    Args:
+        *args: A list of arguments to check for None.
+
+    Returns:
+        True if any of the arguments are None, False otherwise.
+    """
+    return all(arg is None for arg in args)
+
+
+def override[T](*args: T | tuple[T] | list[T] | None) -> list[T] | None:
+    for arg in args:
+        if arg is not None:
+            return normalize(arg)
+    return None
